/* SYSTEM colors */
#vt100 .ansiDef   { color:            inherit; }

#vt100 .ansi0     { color:            inherit; }
#vt100 .ansi1     { color:            inherit; }
#vt100 .ansi2     { color:            inherit; }
#vt100 .ansi3     { color:            inherit; }
#vt100 .ansi4     { color:            inherit; }
#vt100 .ansi5     { color:            inherit; }
#vt100 .ansi6     { color:            inherit; }
#vt100 .ansi7     { color:            inherit; }
#vt100 .ansi8     { color:            inherit; }
#vt100 .ansi9     { color:            inherit; }
#vt100 .ansi10    { color:            inherit; }
#vt100 .ansi11    { color:            inherit; }
#vt100 .ansi12    { color:            inherit; }
#vt100 .ansi13    { color:            inherit; }
#vt100 .ansi14    { color:            inherit; }
#vt100 .ansi15    { color:            inherit; }

/* XTERM colors - 256 color mode */
#vt100 .ansi16    { color:            inherit; }
#vt100 .ansi17    { color:            inherit; }
#vt100 .ansi18    { color:            inherit; }
#vt100 .ansi19    { color:            inherit; }
#vt100 .ansi20    { color:            inherit; }
#vt100 .ansi21    { color:            inherit; }
#vt100 .ansi22    { color:            inherit; }
#vt100 .ansi23    { color:            inherit; }
#vt100 .ansi24    { color:            inherit; }
#vt100 .ansi25    { color:            inherit; }
#vt100 .ansi26    { color:            inherit; }
#vt100 .ansi27    { color:            inherit; }
#vt100 .ansi28    { color:            inherit; }
#vt100 .ansi29    { color:            inherit; }
#vt100 .ansi30    { color:            inherit; }
#vt100 .ansi31    { color:            inherit; }
#vt100 .ansi32    { color:            inherit; }
#vt100 .ansi33    { color:            inherit; }
#vt100 .ansi34    { color:            inherit; }
#vt100 .ansi35    { color:            inherit; }
#vt100 .ansi36    { color:            inherit; }
#vt100 .ansi37    { color:            inherit; }
#vt100 .ansi38    { color:            inherit; }
#vt100 .ansi39    { color:            inherit; }
#vt100 .ansi40    { color:            inherit; }
#vt100 .ansi41    { color:            inherit; }
#vt100 .ansi42    { color:            inherit; }
#vt100 .ansi43    { color:            inherit; }
#vt100 .ansi44    { color:            inherit; }
#vt100 .ansi45    { color:            inherit; }
#vt100 .ansi46    { color:            inherit; }
#vt100 .ansi47    { color:            inherit; }
#vt100 .ansi48    { color:            inherit; }
#vt100 .ansi49    { color:            inherit; }
#vt100 .ansi50    { color:            inherit; }
#vt100 .ansi51    { color:            inherit; }
#vt100 .ansi52    { color:            inherit; }
#vt100 .ansi53    { color:            inherit; }
#vt100 .ansi54    { color:            inherit; }
#vt100 .ansi55    { color:            inherit; }
#vt100 .ansi56    { color:            inherit; }
#vt100 .ansi57    { color:            inherit; }
#vt100 .ansi58    { color:            inherit; }
#vt100 .ansi59    { color:            inherit; }
#vt100 .ansi60    { color:            inherit; }
#vt100 .ansi61    { color:            inherit; }
#vt100 .ansi62    { color:            inherit; }
#vt100 .ansi63    { color:            inherit; }
#vt100 .ansi64    { color:            inherit; }
#vt100 .ansi65    { color:            inherit; }
#vt100 .ansi66    { color:            inherit; }
#vt100 .ansi67    { color:            inherit; }
#vt100 .ansi68    { color:            inherit; }
#vt100 .ansi69    { color:            inherit; }
#vt100 .ansi70    { color:            inherit; }
#vt100 .ansi71    { color:            inherit; }
#vt100 .ansi72    { color:            inherit; }
#vt100 .ansi73    { color:            inherit; }
#vt100 .ansi74    { color:            inherit; }
#vt100 .ansi75    { color:            inherit; }
#vt100 .ansi76    { color:            inherit; }
#vt100 .ansi77    { color:            inherit; }
#vt100 .ansi78    { color:            inherit; }
#vt100 .ansi79    { color:            inherit; }
#vt100 .ansi80    { color:            inherit; }
#vt100 .ansi81    { color:            inherit; }
#vt100 .ansi82    { color:            inherit; }
#vt100 .ansi83    { color:            inherit; }
#vt100 .ansi84    { color:            inherit; }
#vt100 .ansi85    { color:            inherit; }
#vt100 .ansi86    { color:            inherit; }
#vt100 .ansi87    { color:            inherit; }
#vt100 .ansi88    { color:            inherit; }
#vt100 .ansi89    { color:            inherit; }
#vt100 .ansi90    { color:            inherit; }
#vt100 .ansi91    { color:            inherit; }
#vt100 .ansi92    { color:            inherit; }
#vt100 .ansi93    { color:            inherit; }
#vt100 .ansi94    { color:            inherit; }
#vt100 .ansi95    { color:            inherit; }
#vt100 .ansi96    { color:            inherit; }
#vt100 .ansi97    { color:            inherit; }
#vt100 .ansi98    { color:            inherit; }
#vt100 .ansi99    { color:            inherit; }
#vt100 .ansi100   { color:            inherit; }
#vt100 .ansi101   { color:            inherit; }
#vt100 .ansi102   { color:            inherit; }
#vt100 .ansi103   { color:            inherit; }
#vt100 .ansi104   { color:            inherit; }
#vt100 .ansi105   { color:            inherit; }
#vt100 .ansi106   { color:            inherit; }
#vt100 .ansi107   { color:            inherit; }
#vt100 .ansi108   { color:            inherit; }
#vt100 .ansi109   { color:            inherit; }
#vt100 .ansi110   { color:            inherit; }
#vt100 .ansi111   { color:            inherit; }
#vt100 .ansi112   { color:            inherit; }
#vt100 .ansi113   { color:            inherit; }
#vt100 .ansi114   { color:            inherit; }
#vt100 .ansi115   { color:            inherit; }
#vt100 .ansi116   { color:            inherit; }
#vt100 .ansi117   { color:            inherit; }
#vt100 .ansi118   { color:            inherit; }
#vt100 .ansi119   { color:            inherit; }
#vt100 .ansi120   { color:            inherit; }
#vt100 .ansi121   { color:            inherit; }
#vt100 .ansi122   { color:            inherit; }
#vt100 .ansi123   { color:            inherit; }
#vt100 .ansi124   { color:            inherit; }
#vt100 .ansi125   { color:            inherit; }
#vt100 .ansi126   { color:            inherit; }
#vt100 .ansi127   { color:            inherit; }
#vt100 .ansi128   { color:            inherit; }
#vt100 .ansi129   { color:            inherit; }
#vt100 .ansi130   { color:            inherit; }
#vt100 .ansi131   { color:            inherit; }
#vt100 .ansi132   { color:            inherit; }
#vt100 .ansi133   { color:            inherit; }
#vt100 .ansi134   { color:            inherit; }
#vt100 .ansi135   { color:            inherit; }
#vt100 .ansi136   { color:            inherit; }
#vt100 .ansi137   { color:            inherit; }
#vt100 .ansi138   { color:            inherit; }
#vt100 .ansi139   { color:            inherit; }
#vt100 .ansi140   { color:            inherit; }
#vt100 .ansi141   { color:            inherit; }
#vt100 .ansi142   { color:            inherit; }
#vt100 .ansi143   { color:            inherit; }
#vt100 .ansi144   { color:            inherit; }
#vt100 .ansi145   { color:            inherit; }
#vt100 .ansi146   { color:            inherit; }
#vt100 .ansi147   { color:            inherit; }
#vt100 .ansi148   { color:            inherit; }
#vt100 .ansi149   { color:            inherit; }
#vt100 .ansi150   { color:            inherit; }
#vt100 .ansi151   { color:            inherit; }
#vt100 .ansi152   { color:            inherit; }
#vt100 .ansi153   { color:            inherit; }
#vt100 .ansi154   { color:            inherit; }
#vt100 .ansi155   { color:            inherit; }
#vt100 .ansi156   { color:            inherit; }
#vt100 .ansi157   { color:            inherit; }
#vt100 .ansi158   { color:            inherit; }
#vt100 .ansi159   { color:            inherit; }
#vt100 .ansi160   { color:            inherit; }
#vt100 .ansi161   { color:            inherit; }
#vt100 .ansi162   { color:            inherit; }
#vt100 .ansi163   { color:            inherit; }
#vt100 .ansi164   { color:            inherit; }
#vt100 .ansi165   { color:            inherit; }
#vt100 .ansi166   { color:            inherit; }
#vt100 .ansi167   { color:            inherit; }
#vt100 .ansi168   { color:            inherit; }
#vt100 .ansi169   { color:            inherit; }
#vt100 .ansi170   { color:            inherit; }
#vt100 .ansi171   { color:            inherit; }
#vt100 .ansi172   { color:            inherit; }
#vt100 .ansi173   { color:            inherit; }
#vt100 .ansi174   { color:            inherit; }
#vt100 .ansi175   { color:            inherit; }
#vt100 .ansi176   { color:            inherit; }
#vt100 .ansi177   { color:            inherit; }
#vt100 .ansi178   { color:            inherit; }
#vt100 .ansi179   { color:            inherit; }
#vt100 .ansi180   { color:            inherit; }
#vt100 .ansi181   { color:            inherit; }
#vt100 .ansi182   { color:            inherit; }
#vt100 .ansi183   { color:            inherit; }
#vt100 .ansi184   { color:            inherit; }
#vt100 .ansi185   { color:            inherit; }
#vt100 .ansi186   { color:            inherit; }
#vt100 .ansi187   { color:            inherit; }
#vt100 .ansi188   { color:            inherit; }
#vt100 .ansi189   { color:            inherit; }
#vt100 .ansi190   { color:            inherit; }
#vt100 .ansi191   { color:            inherit; }
#vt100 .ansi192   { color:            inherit; }
#vt100 .ansi193   { color:            inherit; }
#vt100 .ansi194   { color:            inherit; }
#vt100 .ansi195   { color:            inherit; }
#vt100 .ansi196   { color:            inherit; }
#vt100 .ansi197   { color:            inherit; }
#vt100 .ansi198   { color:            inherit; }
#vt100 .ansi199   { color:            inherit; }
#vt100 .ansi200   { color:            inherit; }
#vt100 .ansi201   { color:            inherit; }
#vt100 .ansi202   { color:            inherit; }
#vt100 .ansi203   { color:            inherit; }
#vt100 .ansi204   { color:            inherit; }
#vt100 .ansi205   { color:            inherit; }
#vt100 .ansi206   { color:            inherit; }
#vt100 .ansi207   { color:            inherit; }
#vt100 .ansi208   { color:            inherit; }
#vt100 .ansi209   { color:            inherit; }
#vt100 .ansi210   { color:            inherit; }
#vt100 .ansi211   { color:            inherit; }
#vt100 .ansi212   { color:            inherit; }
#vt100 .ansi213   { color:            inherit; }
#vt100 .ansi214   { color:            inherit; }
#vt100 .ansi215   { color:            inherit; }
#vt100 .ansi216   { color:            inherit; }
#vt100 .ansi217   { color:            inherit; }
#vt100 .ansi218   { color:            inherit; }
#vt100 .ansi219   { color:            inherit; }
#vt100 .ansi220   { color:            inherit; }
#vt100 .ansi221   { color:            inherit; }
#vt100 .ansi222   { color:            inherit; }
#vt100 .ansi223   { color:            inherit; }
#vt100 .ansi224   { color:            inherit; }
#vt100 .ansi225   { color:            inherit; }
#vt100 .ansi226   { color:            inherit; }
#vt100 .ansi227   { color:            inherit; }
#vt100 .ansi228   { color:            inherit; }
#vt100 .ansi229   { color:            inherit; }
#vt100 .ansi230   { color:            inherit; }
#vt100 .ansi231   { color:            inherit; }
#vt100 .ansi232   { color:            inherit; }
#vt100 .ansi233   { color:            inherit; }
#vt100 .ansi234   { color:            inherit; }
#vt100 .ansi235   { color:            inherit; }
#vt100 .ansi236   { color:            inherit; }
#vt100 .ansi237   { color:            inherit; }
#vt100 .ansi238   { color:            inherit; }
#vt100 .ansi239   { color:            inherit; }
#vt100 .ansi240   { color:            inherit; }
#vt100 .ansi241   { color:            inherit; }
#vt100 .ansi242   { color:            inherit; }
#vt100 .ansi243   { color:            inherit; }
#vt100 .ansi244   { color:            inherit; }
#vt100 .ansi245   { color:            inherit; }
#vt100 .ansi246   { color:            inherit; }
#vt100 .ansi247   { color:            inherit; }
#vt100 .ansi248   { color:            inherit; }
#vt100 .ansi249   { color:            inherit; }
#vt100 .ansi250   { color:            inherit; }
#vt100 .ansi251   { color:            inherit; }
#vt100 .ansi252   { color:            inherit; }
#vt100 .ansi253   { color:            inherit; }
#vt100 .ansi254   { color:            inherit; }
#vt100 .ansi255   { color:            inherit; }

/* SYSTEM colors */
#vt100 .bgAnsiDef { background-color: transparent; }

#vt100 .bgAnsi0   { background-color: transparent; }
#vt100 .bgAnsi1   { background-color: transparent; }
#vt100 .bgAnsi2   { background-color: transparent; }
#vt100 .bgAnsi3   { background-color: transparent; }
#vt100 .bgAnsi4   { background-color: transparent; }
#vt100 .bgAnsi5   { background-color: transparent; }
#vt100 .bgAnsi6   { background-color: transparent; }
#vt100 .bgAnsi7   { background-color: transparent; }
#vt100 .bgAnsi8   { background-color: transparent; }
#vt100 .bgAnsi9   { background-color: transparent; }
#vt100 .bgAnsi10  { background-color: transparent; }
#vt100 .bgAnsi11  { background-color: transparent; }
#vt100 .bgAnsi12  { background-color: transparent; }
#vt100 .bgAnsi13  { background-color: transparent; }
#vt100 .bgAnsi14  { background-color: transparent; }
#vt100 .bgAnsi15  { background-color: transparent; }

/* XTERM colors - 256 color mode */
#vt100 .bgAnsi16  { background-color: transparent; }
#vt100 .bgAnsi17  { background-color: transparent; }
#vt100 .bgAnsi18  { background-color: transparent; }
#vt100 .bgAnsi19  { background-color: transparent; }
#vt100 .bgAnsi20  { background-color: transparent; }
#vt100 .bgAnsi21  { background-color: transparent; }
#vt100 .bgAnsi22  { background-color: transparent; }
#vt100 .bgAnsi23  { background-color: transparent; }
#vt100 .bgAnsi24  { background-color: transparent; }
#vt100 .bgAnsi25  { background-color: transparent; }
#vt100 .bgAnsi26  { background-color: transparent; }
#vt100 .bgAnsi27  { background-color: transparent; }
#vt100 .bgAnsi28  { background-color: transparent; }
#vt100 .bgAnsi29  { background-color: transparent; }
#vt100 .bgAnsi30  { background-color: transparent; }
#vt100 .bgAnsi31  { background-color: transparent; }
#vt100 .bgAnsi32  { background-color: transparent; }
#vt100 .bgAnsi33  { background-color: transparent; }
#vt100 .bgAnsi34  { background-color: transparent; }
#vt100 .bgAnsi35  { background-color: transparent; }
#vt100 .bgAnsi36  { background-color: transparent; }
#vt100 .bgAnsi37  { background-color: transparent; }
#vt100 .bgAnsi38  { background-color: transparent; }
#vt100 .bgAnsi39  { background-color: transparent; }
#vt100 .bgAnsi40  { background-color: transparent; }
#vt100 .bgAnsi41  { background-color: transparent; }
#vt100 .bgAnsi42  { background-color: transparent; }
#vt100 .bgAnsi43  { background-color: transparent; }
#vt100 .bgAnsi44  { background-color: transparent; }
#vt100 .bgAnsi45  { background-color: transparent; }
#vt100 .bgAnsi46  { background-color: transparent; }
#vt100 .bgAnsi47  { background-color: transparent; }
#vt100 .bgAnsi48  { background-color: transparent; }
#vt100 .bgAnsi49  { background-color: transparent; }
#vt100 .bgAnsi50  { background-color: transparent; }
#vt100 .bgAnsi51  { background-color: transparent; }
#vt100 .bgAnsi52  { background-color: transparent; }
#vt100 .bgAnsi53  { background-color: transparent; }
#vt100 .bgAnsi54  { background-color: transparent; }
#vt100 .bgAnsi55  { background-color: transparent; }
#vt100 .bgAnsi56  { background-color: transparent; }
#vt100 .bgAnsi57  { background-color: transparent; }
#vt100 .bgAnsi58  { background-color: transparent; }
#vt100 .bgAnsi59  { background-color: transparent; }
#vt100 .bgAnsi60  { background-color: transparent; }
#vt100 .bgAnsi61  { background-color: transparent; }
#vt100 .bgAnsi62  { background-color: transparent; }
#vt100 .bgAnsi63  { background-color: transparent; }
#vt100 .bgAnsi64  { background-color: transparent; }
#vt100 .bgAnsi65  { background-color: transparent; }
#vt100 .bgAnsi66  { background-color: transparent; }
#vt100 .bgAnsi67  { background-color: transparent; }
#vt100 .bgAnsi68  { background-color: transparent; }
#vt100 .bgAnsi69  { background-color: transparent; }
#vt100 .bgAnsi70  { background-color: transparent; }
#vt100 .bgAnsi71  { background-color: transparent; }
#vt100 .bgAnsi72  { background-color: transparent; }
#vt100 .bgAnsi73  { background-color: transparent; }
#vt100 .bgAnsi74  { background-color: transparent; }
#vt100 .bgAnsi75  { background-color: transparent; }
#vt100 .bgAnsi76  { background-color: transparent; }
#vt100 .bgAnsi77  { background-color: transparent; }
#vt100 .bgAnsi78  { background-color: transparent; }
#vt100 .bgAnsi79  { background-color: transparent; }
#vt100 .bgAnsi80  { background-color: transparent; }
#vt100 .bgAnsi81  { background-color: transparent; }
#vt100 .bgAnsi82  { background-color: transparent; }
#vt100 .bgAnsi83  { background-color: transparent; }
#vt100 .bgAnsi84  { background-color: transparent; }
#vt100 .bgAnsi85  { background-color: transparent; }
#vt100 .bgAnsi86  { background-color: transparent; }
#vt100 .bgAnsi87  { background-color: transparent; }
#vt100 .bgAnsi88  { background-color: transparent; }
#vt100 .bgAnsi89  { background-color: transparent; }
#vt100 .bgAnsi90  { background-color: transparent; }
#vt100 .bgAnsi91  { background-color: transparent; }
#vt100 .bgAnsi92  { background-color: transparent; }
#vt100 .bgAnsi93  { background-color: transparent; }
#vt100 .bgAnsi94  { background-color: transparent; }
#vt100 .bgAnsi95  { background-color: transparent; }
#vt100 .bgAnsi96  { background-color: transparent; }
#vt100 .bgAnsi97  { background-color: transparent; }
#vt100 .bgAnsi98  { background-color: transparent; }
#vt100 .bgAnsi99  { background-color: transparent; }
#vt100 .bgAnsi100 { background-color: transparent; }
#vt100 .bgAnsi101 { background-color: transparent; }
#vt100 .bgAnsi102 { background-color: transparent; }
#vt100 .bgAnsi103 { background-color: transparent; }
#vt100 .bgAnsi104 { background-color: transparent; }
#vt100 .bgAnsi105 { background-color: transparent; }
#vt100 .bgAnsi106 { background-color: transparent; }
#vt100 .bgAnsi107 { background-color: transparent; }
#vt100 .bgAnsi108 { background-color: transparent; }
#vt100 .bgAnsi109 { background-color: transparent; }
#vt100 .bgAnsi110 { background-color: transparent; }
#vt100 .bgAnsi111 { background-color: transparent; }
#vt100 .bgAnsi112 { background-color: transparent; }
#vt100 .bgAnsi113 { background-color: transparent; }
#vt100 .bgAnsi114 { background-color: transparent; }
#vt100 .bgAnsi115 { background-color: transparent; }
#vt100 .bgAnsi116 { background-color: transparent; }
#vt100 .bgAnsi117 { background-color: transparent; }
#vt100 .bgAnsi118 { background-color: transparent; }
#vt100 .bgAnsi119 { background-color: transparent; }
#vt100 .bgAnsi120 { background-color: transparent; }
#vt100 .bgAnsi121 { background-color: transparent; }
#vt100 .bgAnsi122 { background-color: transparent; }
#vt100 .bgAnsi123 { background-color: transparent; }
#vt100 .bgAnsi124 { background-color: transparent; }
#vt100 .bgAnsi125 { background-color: transparent; }
#vt100 .bgAnsi126 { background-color: transparent; }
#vt100 .bgAnsi127 { background-color: transparent; }
#vt100 .bgAnsi128 { background-color: transparent; }
#vt100 .bgAnsi129 { background-color: transparent; }
#vt100 .bgAnsi130 { background-color: transparent; }
#vt100 .bgAnsi131 { background-color: transparent; }
#vt100 .bgAnsi132 { background-color: transparent; }
#vt100 .bgAnsi133 { background-color: transparent; }
#vt100 .bgAnsi134 { background-color: transparent; }
#vt100 .bgAnsi135 { background-color: transparent; }
#vt100 .bgAnsi136 { background-color: transparent; }
#vt100 .bgAnsi137 { background-color: transparent; }
#vt100 .bgAnsi138 { background-color: transparent; }
#vt100 .bgAnsi139 { background-color: transparent; }
#vt100 .bgAnsi140 { background-color: transparent; }
#vt100 .bgAnsi141 { background-color: transparent; }
#vt100 .bgAnsi142 { background-color: transparent; }
#vt100 .bgAnsi143 { background-color: transparent; }
#vt100 .bgAnsi144 { background-color: transparent; }
#vt100 .bgAnsi145 { background-color: transparent; }
#vt100 .bgAnsi146 { background-color: transparent; }
#vt100 .bgAnsi147 { background-color: transparent; }
#vt100 .bgAnsi148 { background-color: transparent; }
#vt100 .bgAnsi149 { background-color: transparent; }
#vt100 .bgAnsi150 { background-color: transparent; }
#vt100 .bgAnsi151 { background-color: transparent; }
#vt100 .bgAnsi152 { background-color: transparent; }
#vt100 .bgAnsi153 { background-color: transparent; }
#vt100 .bgAnsi154 { background-color: transparent; }
#vt100 .bgAnsi155 { background-color: transparent; }
#vt100 .bgAnsi156 { background-color: transparent; }
#vt100 .bgAnsi157 { background-color: transparent; }
#vt100 .bgAnsi158 { background-color: transparent; }
#vt100 .bgAnsi159 { background-color: transparent; }
#vt100 .bgAnsi160 { background-color: transparent; }
#vt100 .bgAnsi161 { background-color: transparent; }
#vt100 .bgAnsi162 { background-color: transparent; }
#vt100 .bgAnsi163 { background-color: transparent; }
#vt100 .bgAnsi164 { background-color: transparent; }
#vt100 .bgAnsi165 { background-color: transparent; }
#vt100 .bgAnsi166 { background-color: transparent; }
#vt100 .bgAnsi167 { background-color: transparent; }
#vt100 .bgAnsi168 { background-color: transparent; }
#vt100 .bgAnsi169 { background-color: transparent; }
#vt100 .bgAnsi170 { background-color: transparent; }
#vt100 .bgAnsi171 { background-color: transparent; }
#vt100 .bgAnsi172 { background-color: transparent; }
#vt100 .bgAnsi173 { background-color: transparent; }
#vt100 .bgAnsi174 { background-color: transparent; }
#vt100 .bgAnsi175 { background-color: transparent; }
#vt100 .bgAnsi176 { background-color: transparent; }
#vt100 .bgAnsi177 { background-color: transparent; }
#vt100 .bgAnsi178 { background-color: transparent; }
#vt100 .bgAnsi179 { background-color: transparent; }
#vt100 .bgAnsi180 { background-color: transparent; }
#vt100 .bgAnsi181 { background-color: transparent; }
#vt100 .bgAnsi182 { background-color: transparent; }
#vt100 .bgAnsi183 { background-color: transparent; }
#vt100 .bgAnsi184 { background-color: transparent; }
#vt100 .bgAnsi185 { background-color: transparent; }
#vt100 .bgAnsi186 { background-color: transparent; }
#vt100 .bgAnsi187 { background-color: transparent; }
#vt100 .bgAnsi188 { background-color: transparent; }
#vt100 .bgAnsi189 { background-color: transparent; }
#vt100 .bgAnsi190 { background-color: transparent; }
#vt100 .bgAnsi191 { background-color: transparent; }
#vt100 .bgAnsi192 { background-color: transparent; }
#vt100 .bgAnsi193 { background-color: transparent; }
#vt100 .bgAnsi194 { background-color: transparent; }
#vt100 .bgAnsi195 { background-color: transparent; }
#vt100 .bgAnsi196 { background-color: transparent; }
#vt100 .bgAnsi197 { background-color: transparent; }
#vt100 .bgAnsi198 { background-color: transparent; }
#vt100 .bgAnsi199 { background-color: transparent; }
#vt100 .bgAnsi200 { background-color: transparent; }
#vt100 .bgAnsi201 { background-color: transparent; }
#vt100 .bgAnsi202 { background-color: transparent; }
#vt100 .bgAnsi203 { background-color: transparent; }
#vt100 .bgAnsi204 { background-color: transparent; }
#vt100 .bgAnsi205 { background-color: transparent; }
#vt100 .bgAnsi206 { background-color: transparent; }
#vt100 .bgAnsi207 { background-color: transparent; }
#vt100 .bgAnsi208 { background-color: transparent; }
#vt100 .bgAnsi209 { background-color: transparent; }
#vt100 .bgAnsi210 { background-color: transparent; }
#vt100 .bgAnsi211 { background-color: transparent; }
#vt100 .bgAnsi212 { background-color: transparent; }
#vt100 .bgAnsi213 { background-color: transparent; }
#vt100 .bgAnsi214 { background-color: transparent; }
#vt100 .bgAnsi215 { background-color: transparent; }
#vt100 .bgAnsi216 { background-color: transparent; }
#vt100 .bgAnsi217 { background-color: transparent; }
#vt100 .bgAnsi218 { background-color: transparent; }
#vt100 .bgAnsi219 { background-color: transparent; }
#vt100 .bgAnsi220 { background-color: transparent; }
#vt100 .bgAnsi221 { background-color: transparent; }
#vt100 .bgAnsi222 { background-color: transparent; }
#vt100 .bgAnsi223 { background-color: transparent; }
#vt100 .bgAnsi224 { background-color: transparent; }
#vt100 .bgAnsi225 { background-color: transparent; }
#vt100 .bgAnsi226 { background-color: transparent; }
#vt100 .bgAnsi227 { background-color: transparent; }
#vt100 .bgAnsi228 { background-color: transparent; }
#vt100 .bgAnsi229 { background-color: transparent; }
#vt100 .bgAnsi230 { background-color: transparent; }
#vt100 .bgAnsi231 { background-color: transparent; }
#vt100 .bgAnsi232 { background-color: transparent; }
#vt100 .bgAnsi233 { background-color: transparent; }
#vt100 .bgAnsi234 { background-color: transparent; }
#vt100 .bgAnsi235 { background-color: transparent; }
#vt100 .bgAnsi236 { background-color: transparent; }
#vt100 .bgAnsi237 { background-color: transparent; }
#vt100 .bgAnsi238 { background-color: transparent; }
#vt100 .bgAnsi239 { background-color: transparent; }
#vt100 .bgAnsi240 { background-color: transparent; }
#vt100 .bgAnsi241 { background-color: transparent; }
#vt100 .bgAnsi242 { background-color: transparent; }
#vt100 .bgAnsi243 { background-color: transparent; }
#vt100 .bgAnsi244 { background-color: transparent; }
#vt100 .bgAnsi245 { background-color: transparent; }
#vt100 .bgAnsi246 { background-color: transparent; }
#vt100 .bgAnsi247 { background-color: transparent; }
#vt100 .bgAnsi248 { background-color: transparent; }
#vt100 .bgAnsi249 { background-color: transparent; }
#vt100 .bgAnsi250 { background-color: transparent; }
#vt100 .bgAnsi251 { background-color: transparent; }
#vt100 .bgAnsi252 { background-color: transparent; }
#vt100 .bgAnsi253 { background-color: transparent; }
#vt100 .bgAnsi254 { background-color: transparent; }
#vt100 .bgAnsi255 { background-color: transparent; }
