/* Set styles used when opening the printer window */
#print {
  margin: 0.5ex 0px 0px 0px;
}

#options, #spacer {
  font-family:      sans;
  font-size:        smaller;
  position:         fixed;
  top:              0px;
  left:             0px;
  right:            0px;
  background-color: #EEEEEE;
  border-style:     solid;
  border-top:       0px solid black;
  border-left:      0px solid black;
  border-right:     0px solid black;
  border-bottom:    1px solid black;
}

#spacer {
  position:   static;
  visibility: hidden;
}

@media print {
  body {
    margin: 0px;
  }

  #print {
    margin: 0px;
  }

  #print .pagebreak {
    page-break-before: always;
  }

  #print .pagebreak hr, #options {
    display: none;
  }
}
